@@ -1,8 +1,15 @@
-from .metadata import CRAIGSLIST, REGIONS, COUNTRIES, SITES, AREAS
+from .metadata import CRAIGSLIST, REGIONS, COUNTRIES, SITES
+
+
+class classproperty(property):
+    """Allow property attribute for class methods."""
+
+    def __get__(self, obj, objtype=None):
+        return super(classproperty, self).__get__(objtype)
 
 
 class Base:
-    """ Base class for Region, Country, Site, and Area in api.py. """
+    """Base class for Region, Country, Site, and Area in api.py."""
 
     _selector_key = ""
     _subclass = None
@@ -10,13 +17,13 @@ class Base:
         "region": REGIONS,
         "country": COUNTRIES,
         "site": SITES,
-        "area": AREAS,
     }
 
     def __init__(self, key):
         if key not in self._valid_keys[self._selector_key]:
             raise ValueError(
-                "invalid key: '%s'. See list of valid keys using the 'children' attribute" % key
+                "invalid key: '%s'. See list of valid keys using the 'get_keys' method --> %s.get_keys()"
+                % (key, self.__class__.__name__)
             )
         self._key = key
 
@@ -24,53 +31,63 @@ class Base:
         return "%s('%s')" % (self.__class__.__name__, self._key)
 
     def __iter__(self):
-        """ Yield instance(s) of caller's subclass. Sublass instances are instances
-        of caller's children. """
+        """Yield instance(s) of caller's subclass. Sublass instances are instances
+        of caller's children."""
         yield from (
-            self._subclass(child)
-            for child in find_children(CRAIGSLIST, self._selector_key, self._key)
+            self._subclass(child_key)
+            for child_key in find_children(CRAIGSLIST, self._selector_key, self._key)
         )
 
     @classmethod
     def all(cls):
-        """ Yield all instances of current class. """
-        yield from (cls(child) for child in find_all(CRAIGSLIST, cls._selector_key))
+        """Yield all instances of current class."""
+        yield from (cls(key) for key in find_keys(CRAIGSLIST, cls._selector_key))
+
+    @classproperty
+    def keys(cls):
+        """Return supported keys of class."""
+        return list(find_keys(CRAIGSLIST, cls._selector_key))
 
     @property
     def children(self):
-        """ Return children keys of instance. """
+        """Return children keys of instance."""
         return list(find_children(CRAIGSLIST, self._selector_key, self._key))
 
     @property
     def key(self):
-        """ Return key of instance. """
+        """Return key of instance."""
         return self._key
 
     @property
     def title(self):
-        """ Return title of instance. """
+        """Return title of instance."""
         return find_title(CRAIGSLIST, self._selector_key, self._key)
 
     @property
     def url(self):
-        """ Return url of instance. """
+        """Return url of instance."""
         return find_url(CRAIGSLIST, self._selector_key, self._key)
 
 
-def find_all(tree, selector):
-    """ Yield all keys that match tree's 'selector'. """
-    for datum, tree in tree.items():
-        if tree["selector"] == selector:
-            yield datum
-        else:
-            yield from find_all(tree["child"], selector)
+def find_keys(tree, selector):
+    """Yield all keys that match tree's 'selector'."""
+
+    def recurse_keys(tree):
+        for datum, tree in tree.items():
+            """Recurse tree and yield keys in given selector."""
+            if tree["selector"] == selector:
+                yield datum
+            else:
+                yield from recurse_keys(tree["child"])
+
+    yield from sorted(list(set(recurse_keys(tree))))
 
 
 def find_children(tree, selector, key):
-    """ Yield all unique children keys that match tree's 'selector'. """
+    """Yield all unique children keys that match tree's 'selector'."""
 
     def recurse_children(tree):
-        """ Recurse tree and yield selected children. """
+        """Recurse tree and yield selected datum's children."""
         for datum, tree in tree.items():
             if tree["selector"] == selector and datum == key:
                 yield from tree["child"].keys()
@@ -81,10 +98,10 @@ def find_children(tree, selector, key):
 
 
 def find_title(tree, selector, key):
-    """ Return "title" key value that matches tree's "selector" and datum. """
+    """Return "title" key value that matches tree's "selector" and datum."""
 
     def recurse_title(tree):
-        """ Recurse tree and yield selected title. """
+        """Recurse tree and yield selected datum's title."""
         for datum, tree in tree.items():
             if tree["selector"] == selector and datum == key:
                 yield tree["title"]
@@ -95,10 +112,10 @@ def find_title(tree, selector, key):
 
 
 def find_url(tree, selector, key):
-    """ Return url that matches tree's `selector` and datum. """
+    """Return url that matches tree's `selector` and datum."""
 
     def recurse_url(tree, parent=""):
-        """ Recurse tree and yield selected url. """
+        """Recurse tree and yield selected datum's url."""
         for datum, tree in tree.items():
             if tree["selector"] == selector and datum == key:
                 yield build_url(selector, datum, parent)
@@ -109,7 +126,7 @@ def find_url(tree, selector, key):
 
 
 def build_url(selector, child_key, parent_key):
-    """ Return url string that's conditional to `selector` ("site" or "area"). """
+    """Return url string that's conditional to `selector` ("site" or "area")."""
     if selector == "site":
         return "https://%s.craigslist.org/" % child_key
     return "https://%s.craigslist.org/%s/" % (parent_key, child_key)
